package {

import flash.display.Sprite;
import flash.display.BitmapData;
import flash.events.Event;
import flash.events.ActivityEvent;
import flash.utils.Timer;
import flash.events.TimerEvent;

import flash.display.StageAlign;
import flash.display.StageScaleMode;

//video stuff
import flash.media.Camera;
import flash.media.Video;
import flash.geom.Matrix;

/*
Dice Mirror homage to Robert Hodgin and Daniel Rozin
by David Wicks
sansumbrella.com
CC non-commercial share-alike
*/

public class DiceMirror extends Sprite {
	
	private var dice:Array;
	private var w:Number = 64,
				h:Number = 48;
	private var dieSize:Number;
	private var padding:Number = 1;
	private var updateTimer:Timer;
	private var video:Video;
	private var pixels:BitmapData;
	private var mirrorMatrix:Matrix;
	
	public function DiceMirror()
	{
		stage.scaleMode = StageScaleMode.EXACT_FIT;
		stage.align = StageAlign.TOP_LEFT;
		dieSize = 800/w - padding;
		init();
	}
	
	private function init():void
	{		
		pixels = new BitmapData( w, h );
		setupDice();
		setupCamera();
		startTimer();
	}
	
	private function setupDice():void
	{
		dice = new Array();
		
		for( var tx:uint = 0; tx != w; tx++ ){
			for( var ty:uint = 0; ty != h; ty++ ){
				var die = new Die( tx, ty );
				die.width = die.height = dieSize;
				die.x = tx*(die.width + padding);
				die.y = ty*(die.height + padding);
				die.setColor(( 0xFFFFFF >> 16 ));
				dice.push( die );
				addChild( die );
			}
		}
	}
	
	private function setupCamera():void
	{
		mirrorMatrix = new Matrix( -1, 0,
									0, 1);
		mirrorMatrix.tx = w;
		var camera:Camera = Camera.getCamera();
		//trace( Camera.names );
		
		if (camera != null) {
			camera.addEventListener(ActivityEvent.ACTIVITY, cameraHandler);
			camera.setMode( w, h, 18 );
			video = new Video(camera.width, camera.height);
			video.attachCamera(camera);
		} else {
			trace("You need a camera.");
		}	
	}
	
	private function startTimer():void
	{
		updateTimer = new Timer( 50 );
		updateTimer.addEventListener( TimerEvent.TIMER, update, false, 0, true );
		updateTimer.start();
	}
	
	private function update( e:Event ):void
	{
		pixels.draw( video, mirrorMatrix );
		for each( var die:Die in dice ){
			die.setColor( (pixels.getPixel( die.xPos, die.yPos ) >> 16 )); //just grab the red channel
		}
	}
	
	private function cameraHandler( e:ActivityEvent ):void
	{
		trace("DiceMirror::cameraHandler()" + e );
	}
}

}

